function [imbalanceLower, imbalanceUpper] = PaToHComputeImbal(nnzpv, K)
% [imbalanceLower, imbalanceUpper] = PaToHComputeImbal(nnzpv, K)
% given part number for each nnz entry, computes two imbalance metrics for
% K processors.
% imbalanceLower = (minw-meanw)/meanw
% imbalanceUpper = (maxw-meanw)/meanw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of PaToH Matlab Interface
% Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
% ----------------------------------------------------------------------
% PaToH Matlab Interface is released under the GNU Lesser General Public
% License. A copy of the license is included in this distribution in
% COPYING.LESSER file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (nargin < 2)
    K = max(full(max(nnzpv)));
end
mark = zeros(K,1);

[discard1,discard2, pnos] = find(nnzpv);
for j=1:length(pnos),
    if(pnos(j) > K),
        error('in PaToHComputeImbal pnos(%d)=%d, but K=%d\n', j, pnos(j), K);
    end
    if(pnos(j)<=0),
        error('in PaToHComputeImbal pnos(%d)=%d, but should be > 0\n', j, pnos(j));
    end
    mark(pnos(j)) = mark(pnos(j))+1;
end


minW = min(mark);
maxW = max(mark);
meanW = mean(mark);

imbalanceLower = (minW-meanW)/meanW;
imbalanceUpper = (maxW-meanW)/meanW;

if(nargout == 1),
    imbalanceLower = imbalanceUpper;
end
