function [totnumE,maxnumE, totnumF, maxnumF] = PaToHComputeNumber(nnzpv, outpv, inpv, K)
%  [totnumE,maxmumE, totnumF, maxmumF] = ...
%                                 PaToHComputeNumber(nnzpv, outpv, inpv, K)
%
% totnumE,maxnumE, totnumF, maxnumF (total number of messages in Expand,
% maxnumber of messages in expand and so on).
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Some spec for arrays.
% commMatrixExp and commMatrixFld are such that
%           commMatrixXXX(i,j)= V iff processor i sends a message to processor
% j of volume V
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of PaToH Matlab Interface
% Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
% ----------------------------------------------------------------------
% PaToH Matlab Interface is released under the GNU Lesser General Public
% License. A copy of the license is included in this distribution in
% COPYING.LESSER file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin < 3)
    error('Missing arguments: call with at least 3 arguments'); 
end
if (nargin < 4)
    K = max(full(max(nnzpv)));
end
[m,n] = size(nnzpv);
if(length(outpv) ~= m),
    error('length(outpv) ~= m)\n');
end
if(length(inpv) ~= n),
    error('length(inpv) ~= n\n');
end
%%%%%%%%%%%%%%%%EXPAND statistics%%%%%%%%%%%%%%%%%%%%%%%%
totnumE = 0;
maxnumE = 0;
commMatrixExp = zeros(K,K);
for j=1:n,
    mark = zeros(1,K);
    sender = inpv(j);
    mark(sender) = 1;
    [discard1,discard2, pnos] = find(nnzpv(:, j));
    maxpno = max(pnos);
    minpno = min(pnos);
    if(maxpno > K),
        error('in PaToHComputeNumber maxpno=%d, but K=%d\n', maxpno, K);
    end
    if(minpno <= 0),
        error('in PaToHComputeNumber minpno=%d, but should be > 0\n', minpno);
    end
    mark(pnos) = 1;
    commMatrixExp(sender,1:K)= max([commMatrixExp(sender, 1:K); ...
                        mark(1,1:K)], [], 1);
end
for i=1:K,
  commMatrixExp(i,i) = 0;
end

totnumE = sum(sum(commMatrixExp));
maxnumE = max(sum(commMatrixExp,2));

%%%%%%%%%%%%%%%%FOLD statistics%%%%%%%%%%%%%%%%%%%%%%%%
totnumF = 0;
maxnumF = 0;
commMatrixFld = zeros(K,K);
nnzTrans = nnzpv';
for i=1:m,
    mark = zeros(K,1);
    receiver = outpv(i);
    mark(receiver) = 1;    
    [discard1,discard2, pnos] = find(nnzTrans(:, i));
    mark(pnos) = 1;
    commMatrixFld(:,receiver) = max([commMatrixFld(:,receiver) , ...
                        mark(1:K,1)], [], 2);
end
for i=1:K,
  commMatrixFld(i,i) = 0;
end

totnumF = sum(sum(commMatrixFld));
maxnumF=  max(sum(commMatrixFld,2));

if(nargout <= 1),
totnumE = totnumE + totnumF;
end
