function [totvolE,maxvolE, totvolF, maxvolF] = PaToHComputeVolume(nnzpv, outpv, inpv, K)
%[totvolE,maxvolE, totvolF, maxvolF] = PaToHComputeVolume(nnzpv, outpv, inpv, K)
%
%
%commMatrixExp and commMatrixFld are such that
%commMatrixXXX(i,j)= L iff processor i sends a message to processor
%j of length L.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of PaToH Matlab Interface
% Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
% ----------------------------------------------------------------------
% PaToH Matlab Interface is released under the GNU Lesser General Public
% License. A copy of the license is included in this distribution in
% COPYING.LESSER file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if (nargin < 3)
    error('Missing arguments: call with at least 3 arguments'); 
end
[m,n] = size(nnzpv);
if(length(outpv) ~= m),
    error('length(outpv) ~= m)\n');
end
if(length(inpv) ~= n),
    error('length(inpv) ~= n\n');
end
%%%%%%%%%%%%%%%%EXPAND statistics%%%%%%%%%%%%%%%%%%%%%%%%
totvolE = 0;

oumax = max(outpv);
inmax = max(inpv);
nnzmax = max(full(max(nnzpv)));
if (nargin < 4)
    K = nnzmax;
end
if(oumax>K),
   error('called with K=%d but outpv contains %d\n', K, oumax); 
end
if(inmax>K),
   error('called with K=%d but inpv contains %d\n', K, inmax); 
end
if(nnzmax>K),
   error('called with K=%d but nnzpv contains %d\n', K, nnzmax); 
end

oumin = min(outpv);
inmin = min(inpv);
nnzmin = min(full(max(nnzpv)));
if(oumin<=0 ),
   error('outpv contains %d\n', oumin); 
end
if(inmin <=0),
   error('inpv contains %d\n', inmin); 
end
if(nnzmin<=0),
   cdegs = spones(nnzpv')*ones(size(nnzpv,1), 1);
   if (min(cdegs) > 0),
       error('nnzpv contains %d\n', nnzmin);
   end
end

commMatrixExp = zeros(K,K);
for j=1:n,
    mark = zeros(1,K);
    sender = inpv(j);
    mark(sender) = 1;
    [discard1,discard2, pnos] = find(nnzpv(:, j));
    mark(pnos) = 1;
    con = sum(mark);
    totvolE = totvolE + con-1;
    commMatrixExp(sender,1:K)= commMatrixExp(sender, 1:K) + mark(1,1:K);
end
for i=1:K,
  commMatrixExp(i,i)=0;
end
maxvolE = max(sum(commMatrixExp,2));

%%%%%%%%%%%%%%%%FOLD statistics%%%%%%%%%%%%%%%%%%%%%%%%
totvolF = 0; 
maxvolF = 0;
commMatrixFld = zeros(K,K);
nnzTrans = nnzpv';
for i=1:m,
    mark = zeros(K,1);
    receiver = outpv(i);
    mark(receiver) = 1;    
    [discard1,discard2, pnos] = find(nnzTrans(:, i));
    mark(pnos) = 1;
    con = sum(mark);
    totvolF = totvolF + con-1;
    commMatrixFld(:,receiver) = commMatrixFld(:,receiver) + mark(1:K,1);
end
for i=1:K,
  commMatrixFld(i,i) = 0;
end
maxvolF = max(sum(commMatrixFld,2));

buchk = sum(sum(commMatrixExp)) + sum(sum(commMatrixFld));
if(buchk ~= totvolE + totvolF),
  error('totvol does not check\n');
end
if(nargout <= 1),
totvolE = totvolE + totvolF;
end
