function [outpv, inpv] = PaToHNaiveVectorPart(nnzpv, isSym, K)
% [outpv, inpv] = PaToHNaiveVectorPart(nnzpv, isSym)
% given a partition in nnzpv (a sparse matrix) and a nonsymmetric (isSym is 0 on entry) 
%       or a symmetric vector partitioning (isSym is nonzero on entry) is computed
%       and the partitions on the output- and input-vectors are returned in
%       outpv and inpv. 
% Part numbers are between 1 and K (both are inclusive) where K is
%      set to the maximum part number oin nnzpv  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of PaToH Matlab Interface
% Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
% ----------------------------------------------------------------------
% PaToH Matlab Interface is released under the GNU Lesser General Public
% License. A copy of the license is included in this distribution in
% COPYING.LESSER file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[m,n] = size(nnzpv);
outpv = zeros(m, 1);
inpv = zeros(n,1);
if (nargin <= 2),
    K = max(full(max(nnzpv)));
end
if(isSym),
    if(m~=n),
        error('in PaToHNaiveVectorPart, sym vector part is asked but matrix is not square\n');
    end
   
    inpv = spdiags(nnzpv,0);
    missings=find(inpv == 0);
    for i=1:length(missings),
        cno = missings(i);
        [rnos, dummy, connect] = find(nnzpv(:, cno));
        if(isempty(rnos)),
            pno = 1;
        else
            pno = connect(1);
        end
        inpv(cno) = pno;
    end
    outpv=inpv;
else
    pwin = zeros(K,1);
    pwout = zeros(K,1);
    [rid, cid, pid] = find(nnzpv);
    for ii = 1:length(rid),
        rno = rid(ii);
        cno = cid(ii);
        topr = pid(ii);
        if(topr<=0)
            error('topr =%d for %d', topr, ii);
        end
        if(inpv(cno) == 0),
            inpv(cno) = topr;
            pwin(topr) = pwin(topr)+1;
        else
            if((inpv(cno)~= topr) && (pwin(inpv(cno)) > pwin(topr)+1)),
                pwin(inpv(cno)) = pwin(inpv(cno))-1;
                pwin(topr) = pwin(topr)+1;
                inpv(cno) = topr;
            end
        end
        if(outpv(rno) == 0),
            outpv(rno) = topr;
            pwout(topr) = pwout(topr)+1;
        else
            if((outpv(rno)~= topr) &&...
                    (pwout(outpv(rno)) > pwout(topr)+1)),
                pwout(outpv(rno)) = pwout(outpv(rno))-1;
                pwout(topr) = pwout(topr)+1;
                outpv(rno) = topr;
            end
        end
    end
    for i=1:m,
        if(outpv(i) == 0), outpv(i) = mod(i, K) +1; end
    end
    for i=1:n,
        if(inpv(i) == 0), inpv(i) = mod(i, K) +1; end
    end
end

end%function  [outpv, inpv] = PaToHNaiveVectorPart(nnzpv)
