function PaToHSpy(nnzpv, outpv, inpv, K, symb, ttl, printperm, permstyle)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PaToHSpy(nnzpv, [outpv, inpv, K, symb, ttl, printperm])
%PaToHSpy(nnzpv, symb)
%
%  nnzpv     : partno on nnz basis. This is a sparse matrix, with the same
%              sparsity of the input matrix mat.
%  outpv     : the partvector of the output vector of spmxv with mat
%  inpv      : the partvector of the input vector of spmxv with mat
%  K         : the Number of parts. If omitted computed as max(nnzpv).
%  symb      : marker symbol. If it is '?', a different marker is used for
%              each part.  
%  ttl       : title for the plot
%  printperm : if 1, permuted row/column indices are displayes. 
%               We only recommend use of this option for small matrices.
%
% Description: Plots permuted matrix structure returned from 
%              PaToHMatrixPart
%
% See also: PaToHMatrixPart
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is part of PaToH Matlab Interface
% Copyright (c) 2009 Bora Ucar and Umit V. Catalyurek
% ----------------------------------------------------------------------
% PaToH Matlab Interface is released under the GNU Lesser General Public
% License. A copy of the license is included in this distribution in
% COPYING.LESSER file.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   [m,n] = size(nnzpv);
   if (nargin<4)
     K = max(full(max(nnzpv)));
   end
   if (nargin==2)
       symb = outpv;
   elseif (nargin<5)
       symb = '.';
   end
   if(nargin < 3),
       [outpv, inpv] = PaToHNaiveVectorPart(nnzpv, 0, K);       
   end
      
   if (nargin<7)
       printperm = 0;
   end
   
   for cnt=1:K+1
     colors(cnt,:) = plotcolors(cnt);
   end                                                                                                                         
           
   [m n] = size(nnzpv);
    
   if (nargin>2)
       %% find permutations
       inperm = [];
       X = [];
       for cnt= 1:K
            inperm = [inperm; find(inpv==cnt)];
            X = [X; size(inperm)];
       end
       outperm = [];
       Y = [];
       for cnt= 1:K
            outperm = [outperm; find(outpv==cnt)];
            Y = [Y; size(outperm,1)];
       end

       %% compute permuted matrix and get indices and values
       A = nnzpv(outperm, inperm);
   else
       A = nnzpv;
   end
   [i,j,val] = find(A);     
                                                                                                                                                                                          
   
   %% draw the partitioniong
   figure;                                                                                                                     
   hold on; 

   if (nargin>5)
       t = title(ttl);
       set(t, 'FontSize', 14);
   end   

   nz = nnz(nnzpv);
   [tvolE, mvolE, tvolF, mvolF] = PaToHComputeVolume(nnzpv, outpv, inpv, K);
   vol = tvolE + tvolF; 
   [imbalL, imbalH] = PaToHComputeImbal(nnzpv, K);
   imbalL = imbalL*100; imbalH = imbalH*100;
   
   if (K==1)
      xlab = sprintf('nnz = %d', nz);
   else
       % xlab = sprintf('nnz = %d (%.3f%%)\nvol = %d  imbal = [%.1f%%, %.1f%%]', nz,nz*100/(m*n), vol, imbalL, imbalH);
      xlab = sprintf('nnz = %d\nvol = %d  imbal = [%.1f%%, %.1f%%]', nz, vol, imbalL, imbalH);
   end
    
   
   units = get(gca,'units');
   set(gca,'units','points');
   pos = get(gca,'position');
   % markersize = max(4,min(14,round(6*min(pos(3:4))/max(m+1,n+1))))/1.5;
   markersize = max(4,min(14,round(6*min(pos(3:4))/max(m+1,n+1))));
   set(gca,'units',units);
   linestyle = 'none';
   sy = symb;
   for cnt=1:K
       ii = find(val==cnt);
       if (symb=='?')
           sy = plotsymbols(cnt);
       end
       c = colors(cnt,:);
       plot(j(ii), i(ii), 'marker', sy, 'MarkerSize', markersize, ...
           'MarkerFaceColor', c, 'Color', c,'linestyle',linestyle);
       if (nargin>2 && cnt<K)
            if ((Y(cnt)>0) || (Y(cnt)<m))
               line([0, n+1], [Y(cnt)+0.5, Y(cnt)+0.5]);
            end
            if ((X(cnt)>0) && (X(cnt)<n))
               line([X(cnt)+0.5, X(cnt)+0.5], [0, m+1]);
            end
       end
   end
   if (printperm) 
       xtic = []; ytic = [];
       xticl = []; yticl = [];
       for i=1:length(inperm)
           xtic = [xtic; i];
           xticl = [xticl; inperm(i)];
           % text(i, -0.5, int2str(inperm(i)));
       end
       for i=1:length(outperm)
           ytic = [ytic; i];
           yticl = [yticl; outperm(i)];
           % text(n+1.5, i, int2str(outperm(i)));
       end
       set(gca,'YTickLabel', yticl, 'XTickLabel', xticl,...
           'YTick', ytic, 'XTick', xtic);             
   end
   xlabel(xlab, 'FontSize', 12);
   box on;
   set(gca,'xlim',[0 n+1],'ylim',[0 m+1],'ydir','reverse', ...
       'plotboxaspectratio',[n+1 m+1 1]);
   set(gca, 'XGrid', 'off');
   set(gca, 'YGrid', 'off');
   hold off;

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sym = plotsymbols(cindex)
   symlst(1) = 's';
   symlst(2) = '^';
   symlst(3) = 'o';
   symlst(4) = 'd';
   symlst(5) = '.';
   symlst(6) = 'x';
   symlst(7) = 'h';
   symlst(8) = 'p';
   symlst(9) = 'v';
   symlst(10) = '>';
   symlst(11) = '<';
   symlst(12) = '*';
   symlst(13) = '+';
   num = length(symlst);
   sym = symlst(mod(cindex-1,num)+1);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colorV = plotcolors(cindex)
    cmap(1,:) = [ 1 0 0];       % red
    cmap(2,:) = [ 0 1 0];       % green
    cmap(3,:) = [ 0 0 1];       % blue
    cmap(4,:) = [ 1 0 1];       % magenta
    cmap(5,:) = [ 0 1 1];       % cyan
    cmap(6,:) = [ 0 0 0];       % black
    cmap(7,:) = [ 1 0.67 0];    % orange
    cmap(8,:) = [ 0.6 0.6 0.6]; % gray
    cmap(9,:) = [0.75 0.75 0];  % dark yellow
    cmap(10,:) = [ 0.5 0 0];    % dark red
    cmap(11,:) = [ 0 0.4 0];    % dark green
    cmap(12,:) = [ 0 0.0 0.4];  % dark blue
    cmap(13,:) = [ 0.59 1 0.83];% aquamarine
    cmap(14,:) = [ 1 1 0];      % yellow
    cmap(15,:) = [ 0.5 0 0.5];      % dark magenta?
    cmap(16,:) = [ 0 0.5 0.5];      % some green
    
    num=length(cmap(:,1));
    
    colorV = cmap(mod(cindex-1,num)+1,:);
end
